#!/bin/bash



usage() {
  echo
  echo "Usage: ${0} {image} {partition} {mountpoint}"
  echo
  echo "Example: ${0} ./ubuntu.img 2 ./u"
  echo
  exit 1
}


if [ ! ${#} -eq 3 ]
then
  echo "Error: Incorrect number of parameters!"
  usage
fi
if [ ! -f "${1}" ]
then
  echo "Error: File ${1} not found!"
  usage
fi
if [ -z "${2}" ]
then
  echo "Error: Invalid partition ${2}!"
  usage
fi
if [ ! -d "${3}" ]
then
  echo "Error: Mount point ${3} not found!"
  usage
fi

image="${1}"
partition="${2}"
mountpoint="${3}"

partline=`/sbin/fdisk -lu "${image}" | grep "${image}${partition}"`
offset=`echo ${partline} | cut -f2 -d' '`
offsetn=$(( ${offset} + 0 ))
if [ "${offset}" != "${offsetn}" ]
then
  echo "Error: Unable to find offset (found ${offset} in string)"
  echo "       (${partline})"
  exit 1
fi

off=$(( ${offsetn} * 512 ))
mount -o loop,offset=${off} "${image}" "${mountpoint}"
if [ ${?} -ne 0 ]
then
  echo "Error: Failed to mount parition ${partition} at offset ${off}"
  exit 1
fi
exit 0
