#!/bin/bash

# prefix image
# rm /usr/sbin/unattended-*

# Image to key
# sudo dd if=ubuntu-18.04.3-4.14-minimal-odroid-xu4-20190910.img of=/dev/sdc bs=512

# Boot
# root / odroid

####################################################################

declare -A node_ips
node_ips=(
  ['c1']="192.168.1.1"
  ['c2']="192.168.1.2"
  ['c3']="192.168.1.3"
  ['c4']="192.168.1.4"
  ['c5']="192.168.1.5"
  ['c6']="192.168.1.6"
  ['c7']="192.168.1.7"
  ['c8']="192.168.1.8"
  ['c9']="192.168.1.9"
  ['c10']="192.168.1.10"
)
declare -A node_labels
node_labels=(
  ['c1']="A"
  ['c2']="A"
  ['c3']="A"
  ['c4']="A"
  ['c5']="A"
  ['c6']="B"
  ['c7']="B"
  ['c8']="B"
  ['c9']="B"
  ['c10']="B"
)

mfsmaster="192.168.1.111"
lan_gateway="192.168.1.250"
lan_cidr="24"
is_static="false"
ntp_server="192.168.1.250"
name_server="192.168.1.250"
admin_network="192.168.2.0/24"


usage() {
  echo
  echo "Usage: ${0} node-name [master|slave]"
  echo
  echo -n "       Where valid node-name values are:"
  for key in "${!node_ips[@]}"
  do
    echo -n " ${key}"
  done
  echo
  echo
  exit 1
}


failure() {
  echo
  echo "Failure detected, exiting."
  echo
  exit 2
}


do_keyboard() {
  if [ ! -f "/etc/default/keyboard.orig" ]
  then
    echo
    echo "+---- Fixing keyboard layout"
    cp /etc/default/keyboard /etc/default/keyboard.orig
    sed -i "s/gb/us/g" /etc/default/keyboard
  fi
}


do_base_packages() {
  dpkg -l iptables-persistent >/dev/null 2>&1
  if [ ${?} -eq 1 ]
  then
    echo
    echo "+---- Adding base packages"
    export DEBIAN_FRONTEND="noninteractive"
    apt-get update
    [ ${?} -ne 0 ] && failure
    apt-get install -q -y iptables-persistent netfilter-persistent
    [ ${?} -ne 0 ] && failure
    apt-get install -q -y net-tools
    [ ${?} -ne 0 ] && failure
    apt-get install -q -y cryptsetup yubikey-luks
    [ ${?} -ne 0 ] && failure
    sed -i "s#scripts\/#lib\/cryptsetup\/cryptdisks\.#g" /usr/share/yubikey-luks/ykluks-keyscript
    apt-get install -q -y gdisk xfsprogs hdparm hddtemp iperf3 rsync
    [ ${?} -ne 0 ] && failure
  fi
}


do_moosefs_packages() {
  dpkg -l moosefs-* >/dev/null 2>&1
  if [ ${?} -eq 1 ]
  then
    echo
    echo "+---- Adding MooseFS packages"
    # Ubuntu has no armf, so leveraging Raspbian, carefully
    wget -O - "https://ppa.moosefs.com/moosefs.key" | apt-key add -
    # echo "deb http://ppa.moosefs.com/moosefs-3/apt/ubuntu/bionic bionic main" > /etc/apt/sources.list.d/moosefs.list
    echo "deb http://ppa.moosefs.com/moosefs-3/apt/raspbian/stretch stretch main" > /etc/apt/sources.list.d/moosefs.list
    apt-get update
    [ ${?} -ne 0 ] && failure
    apt-get install -q -y moosefs-master moosefs-cgi moosefs-cgiserv moosefs-cli
    [ ${?} -ne 0 ] && failure
    apt-get install -q -y moosefs-chunkserver moosefs-metalogger
    [ ${?} -ne 0 ] && failure
    rm /etc/apt/sources.list.d/moosefs.list
    apt-get update
    [ ${?} -ne 0 ] && failure
  fi
}


do_modules() {
  if [ ! -f "/etc/modprobe.d/xt_recent.conf" ]
  then
    echo
    echo "+---- Tuning/blacklisting kernel modules"
    echo "options xt_recent ip_list_tot=2000 ip_pkt_list_tot=255" > /etc/modprobe.d/xt_recent.conf
    chmod 644 "/etc/modprobe.d/xt_recent.conf"
  fi
}


do_base_services() {
  echo
  echo "+---- Disabling base services"
  systemctl disable ModemManager
  systemctl stop ModemManager
  systemctl disable unattended-upgrades
  systemctl stop unattended-upgrades
  systemctl disable wpa_supplicant
  systemctl stop wpa_supplicant
  systemctl mask apt-daily.service apt-daily-upgrade.service
  systemctl stop apt-daily.timer
  systemctl disable apt-daily.service apt-daily-upgrade.service
  systemctl disable apt-daily.timer apt-daily-upgrade.timer
  if [ -f "/etc/cron.daily/apt-compat" ]
  then
    rm "/etc/cron.daily/apt-compat"
  fi
  if [ -f "/etc/cron.daily/bsdmainutils" ]
  then
    rm "/etc/cron.daily/bsdmainutils"
  fi
  if [ -f "/etc/cron.daily/ubuntu-advantage-tools" ]
  then
    rm "/etc/cron.daily/ubuntu-advantage-tools"
  fi
}


do_moosefs_services() {
  [ ${#} -ne 2 ] && failure
  iam="${1}"
  lbl="${2}"
  echo
  echo "+---- Configuring MooseFS services"
  if [ ! -d "/etc/mfs" ]
  then
    mkdir "/etc/mfs"
  fi
  chown root:root "/etc/mfs"
  chmod 755 "/etc/mfs"
  if [ ! -d /chunks ]
  then
    mkdir /chunks
  fi
  chown root:root /chunks
  chmod 755 /chunks

  grep "mfsmaster" /etc/hosts >/dev/null 2>&1
  if [ ${?} != 0 ]
  then
    echo "${mfsmaster} mfsmaster" >> /etc/hosts
  fi

  # MASTER
  #~~~~~~~~~
  if [ -f "/etc/mfs/mfsexports.cfg.sample" ]
  then
    cp "/etc/mfs/mfsexports.cfg.sample" "/etc/mfs/mfsexports.cfg"
    chmod 644 "/etc/mfs/mfsexports.cfg"
  fi
  if [ -f "/etc/mfs/mfsmaster.cfg.sample" ]
  then
    (
      cat "/etc/mfs/mfsmaster.cfg.sample"
      echo
      echo "### SATURN ###"
      echo "DATA_PATH = /metadata/master"
      echo "NICE_LEVEL = -15"
      echo "# LOCK_MEMORY = 1"
      echo "ATIME_MODE = 4"
    ) > "/etc/mfs/mfsmaster.cfg"
    chmod 644 "/etc/mfs/mfsmaster.cfg"
  fi
  if [ ! -f "/etc/default/moosefs-master" ]
  then
    touch "/etc/default/moosefs-master"
    chmod 644 "/etc/default/moosefs-master"
  fi

  # METALOGGER
  #~~~~~~~~~~~~
  if [ -f "/etc/mfs/mfsmetalogger.cfg.sample" ]
  then
    (
      cat "/etc/mfs/mfsmetalogger.cfg.sample"
      echo
      echo "### SATURN ###"
      echo "DATA_PATH = /metadata/logger"
      echo "NICE_LEVEL = -15"
      echo "META_DOWNLOAD_FREQ = 2"
    ) > "/etc/mfs/mfsmetalogger.cfg"
    chmod 644 "/etc/mfs/mfsmetalogger.cfg"
  fi

  # CGI SERVER
  #~~~~~~~~~~~~
  if [ ! -f "/etc/default/moosefs-cgiserv" ]
  then
    touch "/etc/default/moosefs-cgiserv"
    chmod 644 "/etc/default/moosefs-cgiserv"
  fi

  # CHUNK SERVER
  #~~~~~~~~~~~~~~
  if [ -f "/etc/mfs/mfschunkserver.cfg.sample" ]
  then
    (
      cat "/etc/mfs/mfschunkserver.cfg.sample"
      echo
      echo "### SATURN ###"
      echo "NICE_LEVEL = -10"
      echo "LOCK_MEMORY = 1"
      echo "LABELS = ${lbl}"
    ) > "/etc/mfs/mfschunkserver.cfg"
    chmod 644 "/etc/mfs/mfschunkserver.cfg"
  fi
  if [ -f "/etc/mfs/mfshdd.cfg.sample" ]
  then
    cp "/etc/mfs/mfshdd.cfg.sample" "/etc/mfs/mfshdd.cfg"
    chmod 644 "/etc/mfs/mfshdd.cfg"
  fi

  # Master / Slave
  #~~~~~~~~~~~~~~~~
  if [ "${iam}" == "master" ]
  then
    echo "MFSMASTER_ENABLE=true"      > /etc/default/moosefs-master
    echo "MFSMETALOGGER_ENABLE=false" > /etc/default/moosefs-metalogger
    systemctl disable moosefs-metalogger
    systemctl enable moosefs-master
  else
    echo "MFSMASTER_ENABLE=false"     > /etc/default/moosefs-master
    echo "MFSMETALOGGER_ENABLE=true"  > /etc/default/moosefs-metalogger
    systemctl disable moosefs-master
    systemctl enable moosefs-metalogger
  fi
  echo "MFSCGISERV_ENABLE=true"        > /etc/default/moosefs-cgiserv
  echo "MFSCHUNKSERVER_ENABLE=true"    > /etc/default/moosefs-chunkserver
  systemctl enable moosefs-cgiserv
  systemctl enable moosefs-chunkserver
}


do_script_copies() {
  echo
  echo "+---- Copying other scripts"
  for scr in init-disk mount-all umount-all wireck
  do
    if [ -f "${scr}" ]
    then
      mv "${scr}" "/usr/sbin/${scr}"
      chown root:root "/usr/sbin/${scr}"
      chmod 700 "/usr/sbin/${scr}"
    fi
  done
}


do_kernel() {
  echo
  echo "+---- Kernel Tuning"
  sysctld="/etc/sysctl.d"
  sysctlf="/etc/sysctl.conf"
  if [ -f "${sysctlf}" ]
  then
    rm -f "${sysctlf}" ]
    if [ -f "${sysctlf}" ]
    then
      echo "Warning: the sysctl file could not be removed; ${sysctlf}"
      # failure
    fi
  fi
  touch "${sysctlf}"
  if [ ! -f "${sysctlf}" ]
  then
    echo "Warning: the sysctl file could not be created; ${sysctlf}"
    # failure
  fi
  chmod 644 "${sysctlf}"
  if [ -d "${sysctld}" ]
  then
    rm -rf "${sysctld}" ]
    if [ -d "${sysctld}" ]
    then
      echo "Warning: the sysctl directory could not be removed; ${sysctld}"
      # failure
    fi
  fi
  mkdir "${sysctld}"
  if [ ! -d "${sysctld}" ]
  then
    echo "Warning: the sysctl directory could not be created; ${sysctld}"
    failure
  fi
  chmod 755 "${sysctld}"
  ln -s "${sysctlf}" "${sysctld}/99-sysctl.conf"
  (
    echo "## ARM Tuning"
    echo "kernel.printk = 3 4 1 3"
    echo "vm.min_free_kbytes = 16384"
    echo "## Out-of-the-box Raspbian Hardening"
    echo "fs.protected_hardlinks = 1"
    echo "fs.protected_symlinks = 1"
    echo "## Out-of-the-box Debian Hardening"
    echo "vm.mmap_min_addr = 32768 "
    echo "kernel.kptr_restrict = 1"
    echo "kernel.yama.ptrace_scope = 1"
    echo "net.ipv4.conf.default.rp_filter=1"
    echo "net.ipv4.conf.all.rp_filter=1"
    echo "net.ipv6.conf.all.use_tempaddr = 2"
    echo "net.ipv6.conf.default.use_tempaddr = 2"
    echo "## Security / Hardening"
    echo "kernel.sysrq = 0"
    echo "net.ipv4.conf.all.accept_redirects = 0"
    echo "net.ipv4.conf.all.send_redirects = 0"
    echo "net.ipv6.conf.all.disable_ipv6 = 1"
    echo "net.ipv6.conf.default.disable_ipv6 = 1"
    echo "net.ipv6.conf.lo.disable_ipv6 = 1"
    echo "net.ipv6.conf.eth0.disable_ipv6 = 1"
    echo "net.netfilter.nf_conntrack_tcp_loose = 0"
    echo "## Performance over security"
    echo "net.ipv4.tcp_syncookies = 0"
    echo "net.ipv4.tcp_timestamps = 0"
    echo "## Network Performance"
    echo "net.ipv4.tcp_window_scaling = 1"
    echo "net.core.rmem_max = 268435456"
    echo "net.core.rmem_default = 67108864"
    echo "net.core.wmem_max = 268435456"
    echo "net.core.wmem_default = 67108864"
    echo "net.core.optmem_max = 2097152"
    echo "net.ipv4.tcp_rmem = 1048576 67108864 268435456"
    echo "net.ipv4.tcp_wmem = 1048576 67108864 268435456"
    echo "net.ipv4.tcp_mem = 1048576 67108864 268435456"
    echo "net.ipv4.udp_mem = 1048576 67108864 268435456"
    echo "net.core.netdev_max_backlog = 10000"
    echo "net.ipv4.tcp_no_metrics_save = 1"
    echo "## Memory / Cache Management"
    echo "vm.overcommit_memory = 1"
    echo "# vm.dirty_background_ratio = 3"
    echo "vm.dirty_ratio = 10"
    echo "vm.vfs_cache_pressure = 120"
  ) > "${sysctlf}"
}


do_timesync() {
  if [ ${#ntp_server} -ne 0 ]
  then
    grep "^NTP=" /etc/systemd/timesyncd.conf >/dev/null 2>&1
    if [ ${?} -eq 1 ]
    then
      echo
      echo "+---- Setting NTP server to IP address: ${ntp_server}"
      echo >> /etc/systemd/timesyncd.conf
      echo "NTP=${ntp_server}" >> /etc/systemd/timesyncd.conf
    fi
  fi
  echo
  echo "+---- Enabling network time synchronisation"
  timedatectl set-ntp true 
}


do_nic() {
  [ ${#} -ne 1 ] && failure
  iam="${1}"
  if [ "${is_static}x" == "truex" ]
  then
    echo
    echo "+---- Setting static IP address: ${iam}"
    (
      echo "hostname"
      echo "clientid"
      echo "persistent"
      echo "option rapid_commit"
      echo "option domain_name_servers, domain_name, domain_search, host_name"
      echo "option classless_static_routes"
      echo "option interface_mtu"
      echo "# option ntp_servers"
      echo "require dhcp_server_identifier"
      echo "slaac private"
      echo "interface eth0"
      echo "noipv6"
      echo "nodhcp"
      echo "static ip_address=${iam}/${lan_cidr}"
      echo "static routers=${lan_gateway}"
      echo "static domain_name_servers=${name_server}"
    ) > "/etc/dhcpcd.conf"
  fi
}


do_firewall() {
  echo
  echo "+---- Setting iptables rules"
  (
    echo "*filter"
    echo ":INPUT DROP [0:0]"
    echo ":FORWARD DROP [0:0]"
    echo ":OUTPUT DROP [0:0]"
    echo "COMMIT"
  ) > "/etc/iptables/rules.v6"
  (
    echo "*filter"
    echo ":INPUT DROP [0:0]"
    echo ":FORWARD DROP [0:0]"
    echo ":OUTPUT DROP [0:0]"
    echo "-A INPUT -i lo -j ACCEPT"
    echo "-A INPUT -m state --state RELATED,ESTABLISHED -j ACCEPT"
    echo "-A INPUT -m tcp -p tcp --dport 9400:9450 -m state --state NEW -j ACCEPT"
    echo "-A INPUT -m tcp -p tcp --dport 22 -s ${admin_network} -m state --state NEW -j ACCEPT"
    echo "-A INPUT -m udp -p udp --sport 67:68 --dport 67:68 -d 255.255.255.255 -m state --state NEW -j ACCEPT"
    echo "-A INPUT -d 224.0.0.1 -j DROP"
    echo "-A INPUT -m udp -p udp --dport 137:138 -j DROP"
    echo "-A INPUT -j LOG --log-prefix \"INPUT:DROP:\" --log-level 6"
    echo "-A INPUT -j DROP"
    echo "-A FORWARD -j LOG --log-prefix \"FORWARD:DROP:\" --log-level 6"
    echo "-A FORWARD -j DROP"
    echo "-A OUTPUT -o lo -j ACCEPT"
    echo "-A OUTPUT -m state --state RELATED,ESTABLISHED -j ACCEPT"
    echo "-A OUTPUT -m tcp -p tcp --dport 9400:9450 -m state --state NEW -j ACCEPT"
    echo "-A OUTPUT -m udp -p udp --dport 123 -m state --state NEW -j ACCEPT"
    echo "-A OUTPUT -m udp -p udp --sport 67:68 --dport 67:68 -m state --state NEW -j ACCEPT"
    echo "-A OUTPUT -m udp -p udp --dport 53 -m state --state NEW -j ACCEPT"
    echo "-A OUTPUT -m tcp -p tcp --dport 53 -m state --state NEW -j ACCEPT"
    echo "-A OUTPUT -j LOG --log-prefix \"OUTPUT:DROP:\" --log-level 6"
    echo "-A OUTPUT -j DROP"
    echo "COMMIT"
  ) > "/etc/iptables/rules.v4"
}


do_sshd() {
  echo
  echo "+---- Hardening sshd"
  # fix to prevent root login, aligns to Raspbian
  (
    echo "ChallengeResponseAuthentication no"
    echo "UsePAM yes"
    echo "X11Forwarding no"
    echo "PrintMotd no"
    echo "AcceptEnv LANG LC_*"
    echo "Subsystem	sftp	/usr/lib/openssh/sftp-server"
  ) > "/etc/ssh/sshd_config"
  # systemctl enable ssh
}


do_tty() {
  if [ -f "/etc/systemd/system/getty.target.wants/getty@tty1.service" ]
  then
    echo
    echo "+---- Disabling tty1"
    systemctl disable "getty@tty1"
    rm -f "/etc/systemd/system/getty.target.wants/getty@tty1.service"
  fi
}


do_unbrand() {
  [ ${#} -ne 1 ] && failure
  iam="${1}"
  if [ ! -f "/etc/issue-org" ]
  then
    echo
    echo "+---- Unbranding logon screens"
    cp /etc/issue /etc/issue-orig
    echo > /etc/issue
    cp /etc/issue.net /etc/issue.net-orig
    echo > /etc/issue.net
  fi
  echo "${iam}" > /etc/hostname
  sed -i "s/odroid/${iam}/g" /etc/hosts
  [ ${?} -ne 0 ] && failure
}


do_admin_user() {
  adm="guru"
  echo
  echo "+---- Adding admin account: ${adm}"
  # addgroup "${adm}"
  adduser "${adm}" --gecos ",,," --disabled-password
  # [ ${?} -ne 0 ] && failure
  usermod -a -G sudo "${adm}"
  [ ${?} -ne 0 ] && failure
  echo "passwd(${adm})"
  passwd "${adm}"
  [ ${?} -ne 0 ] && failure
}


do_script_iptdown() {
  scr="/usr/sbin/iptdown"
  if [ ! -x "${scr}" ]
  then
    echo
    echo "+---- Creating script iptdown: ${scr}"
    (
      echo "#!/bin/bash"
      echo
      echo "iptables -P INPUT ACCEPT"
      echo "iptables -P OUTPUT ACCEPT"
      echo "iptables -F"
    ) > "${scr}"
    chown root:root "${scr}"
    chmod 700 "${scr}"
  fi
}


do_script_scheddsk() {
  scr="/usr/sbin/sched-disk"
  if [ ! -x "${scr}" ]
  then
    echo
    echo "+---- Creating script sched-disk: ${scr}"
    (
      echo "#!/bin/bash"
      echo
      echo "for DISK in sda sdb sdc sdd sde sdf sdg sdh"
      echo "do"
      echo "  if [ -b \"/dev/\${DISK}\" ]"
      echo "  then"
      echo "    echo \"+--- Configuring scheduler for /dev/\${DISK}\""
      echo "    echo \"deadline\"   > /sys/block/\${DISK}/queue/scheduler"
      echo "    echo \"24\"         > /sys/block/\${DISK}/queue/iosched/fifo_batch"
      echo "    echo \"750\"        > /sys/block/\${DISK}/queue/iosched/read_expire"
      echo "    echo \"4\"          > /sys/block/\${DISK}/queue/iosched/writes_starved"
      echo "    echo \"128\"        > /sys/block/\${DISK}/queue/read_ahead_kb"
      echo "    echo \"128\"        > /sys/block/\${DISK}/queue/nr_requests"
      echo "  fi"
      echo "done"
    ) > "${scr}"
    chown root:root "${scr}"
    chmod 700 "${scr}"
  fi
}


do_script_tempck() {
  scr="/usr/sbin/tempck"
  if [ ! -x "${scr}" ]
  then
    echo
    echo "+---- Creating script tempck: ${scr}"
    (
      echo "#!/bin/bash"
      echo
      echo "for ZONE in 0 1 2 3 4 5 6 7"
      echo "do"
      echo "  if [ -f \"/sys/devices/virtual/thermal/thermal_zone\${ZONE}/temp\" ]"
      echo "  then"
      echo "    temp=\`cat \"/sys/devices/virtual/thermal/thermal_zone\${ZONE}/temp\"\`"
      echo "    temp=\$(( \${temp} / 1000 ))"
      echo "    alarm=\"\""
      echo "    if [ \${temp} -gt 93 ]"
      echo "    then"
      echo "      alarm=\" <-- warning\""
      echo "    fi"
      echo "    echo \"tz\${ZONE}: \${temp} \${alarm}\""
      echo "  fi"
      echo "done"
      echo "for DISK in sda sdb sdc sdd sde sdf sdg sdh"
      echo "do"
      echo "  if [ -b \"/dev/\${DISK}\" ]"
      echo "  then"
      echo "    temp=\`hddtemp -n -u c SATA:/dev/\${DISK}\`"
      echo "    if [ \${temp} -gt 63 ]"
      echo "    then"
      echo "      alarm=\" <-- warning\""
      echo "    fi"
      echo "    echo \"\${DISK}: \${temp} \${alarm}\""
      echo "  fi"
      echo "done"
    ) > "${scr}"
    chown root:root "${scr}"
    chmod 700 "${scr}"
  fi
}


do_script_tempwatch() {
  scr="/usr/sbin/tempwatch"
  if [ ! -x "${scr}" ]
  then
    echo
    echo "+---- Creating script tempwatch: ${scr}"
    (
      echo "#!/bin/bash"
      echo
      echo "watch -n2 tempck"
    ) > "${scr}"
    chown root:root "${scr}"
    chmod 700 "${scr}"
  fi
}


do_script_cpucap() {
  scr="/usr/sbin/cpucap"
  if [ ! -x "${scr}" ]
  then
    echo
    echo "+---- Creating script cpucap: ${scr}"
    (
      echo "#!/bin/bash"
      echo
      echo "if [ \"\${1}x\" == \"resetx\" -o \"\${1}x\" == \"restorex\" ]"
      echo "then"
      echo "  echo \"1500000\" > /sys/devices/system/cpu/cpu0/cpufreq/scaling_max_freq"
      echo "  echo \"2000000\" > /sys/devices/system/cpu/cpu4/cpufreq/scaling_max_freq"
      echo "  echo \"performance\" > /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor"
      echo "  echo \"performance\" > /sys/devices/system/cpu/cpu4/cpufreq/scaling_governor"
      echo "else"
      echo "  echo \"1000000\" > /sys/devices/system/cpu/cpu0/cpufreq/scaling_max_freq"
      echo "  echo \"1000000\" > /sys/devices/system/cpu/cpu4/cpufreq/scaling_max_freq"
      echo "  echo \"ondemand\" > /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor"
      echo "  echo \"ondemand\" > /sys/devices/system/cpu/cpu4/cpufreq/scaling_governor"
      echo "fi"
      echo
      echo "max0f=\`cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_max_freq\`"
      echo "max4f=\`cat /sys/devices/system/cpu/cpu4/cpufreq/scaling_max_freq\`"
      echo "gov0f=\`cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor\`"
      echo "gov4f=\`cat /sys/devices/system/cpu/cpu4/cpufreq/scaling_governor\`"
      echo "echo \"First CPU, governor and max frequency:  \${gov0f} \${max0f}\""
      echo "echo \"Second CPU, governor and max frequency: \${gov4f} \${max4f}\""
    ) > "${scr}"
    chown root:root "${scr}"
    chmod 700 "${scr}"
  fi
}


do_script_masternic() {
  scr="/usr/sbin/masternic"
  if [ ! -x "${scr}" ]
  then
    echo
    echo "+---- Creating script masternic: ${scr}"
    (
      echo "#!/bin/bash"
      echo
      echo "am_master=0"
      echo "am_slave=0"
      echo
      echo "if [ -f \"/etc/default/moosefs-master\" ]"
      echo "then"
      echo "  grep \"MFSMASTER_ENABLE=true\" \"/etc/default/moosefs-master\" >/dev/null 2>&1"
      echo "  if [ \${?} -eq 0 ]"
      echo "  then"
      echo "    am_master=1"
      echo "  fi"
      echo "fi"
      echo "if [ -f \"/etc/default/moosefs-metalogger\" ]"
      echo "then"
      echo "  grep \"MFSMETALOGGER_ENABLE=true\" \"/etc/default/moosefs-metalogger\" >/dev/null 2>&1"
      echo "  if [ \${?} -eq 0 ]"
      echo "  then"
      echo "    am_slave=1"
      echo "  fi"
      echo "fi"
      echo
      echo "if [ \${am_master} -eq 1 -a \${am_slave} -eq 0 ]"
      echo "then"
      echo "  echo \"Assessment: This node is a master.\""
      echo "  logger \"masternic assessment: This node is a master.\""
      echo "  arping -q -c 3 -f -I eth0 mfsmaster >/dev/null 2>&1"
      echo "  if [ \${?} -eq 0 ]"
      echo "  then"
      echo "    echo \"Error: There is another node responding as master, aborting.\""
      echo "    logger \"masternic error: There is another node responding as master, aborting.\""
      echo "    exit 1"
      echo "  else"
      echo "    echo \"Enabling master interface\""
      echo "    logger \"masternic enabling master interface\""
      echo "    ifconfig eth0:1 mfsmaster netmask 255.255.255.255"
      echo "    exit 0"
      echo "  fi"
      echo "elif [ \${am_master} -eq 0 -a \${am_slave} -eq 1 ]"
      echo "then"
      echo "  echo \"Assessment: This node is a slave.\""
      echo "  logger \"masternic assessment: This node is a slave.\""
      echo "  echo \"Disabling master interface\""
      echo "  logger \"masternic disabling master interface\""
      echo "  ifconfig eth0:1 down >/dev/null 2>&1"
      echo "  exit 0"
      echo "elif [ \${am_master} -eq 1 -a \${am_slave} -eq 1 ]"
      echo "then"
      echo "  echo \"Error: This node is both master and slave, aborting\""
      echo "  logger \"masternic error: This node is both master and slave, aborting\""
      echo "  exit 1"
      echo "elif [ \${am_master} -eq 0 -a \${am_slave} -eq 0 ]"
      echo "then"
      echo "  echo \"Error: This node is neither master nor slave, aborting\""
      echo "  logger \"masternic error: This node is neither master nor slave, aborting\""
      echo "  exit 1"
      echo "fi"
    ) > "${scr}"
    chown root:root "${scr}"
    chmod 700 "${scr}"
  fi
}


do_script_rclocal() {
  scr="/etc/rc.local"
  echo
  echo "+---- Creating script rc.local: ${scr}"
  (
    echo "#!/bin/sh -e"
    echo
    echo "for scr in cpucap masternic sched-disk wireck"
    echo "do"
    echo "  if [ -x \"/usr/sbin/\${scr}\" ]"
    echo "  then"
    echo "    nohup \"/usr/sbin/\${scr}\" < /dev/null > /dev/null 2>&1 &"
    echo "  fi"
    echo "done"
    echo
    echo "exit 0"
  ) > "${scr}"
  chown root:root "${scr}"
  chmod 755 "${scr}"
}



dev=""
if [ ${#} -ne 1 -a ${#} -ne 2 ]
then
  echo "Error: This program requires one or two paramaters."
  usage
elif [ ! ${node_ips[$1]+_} ]
then
  echo "Error: The parameter '${1}' is not recognised."
  usage
elif [ ${#} -eq 2 -a "${2}x" != "masterx" -a "${2}x" != "slavex" ]
then
  echo "Error: The parameter '${2}' is not recognised."
  usage
fi
if [ ${#} -eq 2 ]
then
  thistype="${2}"
else
  thistype="slave"
fi
thishost="${1}"
thisip="${node_ips[$1]}"
thislabel="${node_labels[$1]}"

echo
echo "Building node: ${thishost}"
echo "Building as:   ${thistype}"

do_keyboard
do_base_packages
do_base_services
do_kernel
do_modules
do_timesync
do_nic "${thisip}"
do_firewall
do_moosefs_packages
do_moosefs_services "${thistype}" "${thislabel}"
do_script_iptdown
do_script_scheddsk
do_script_cpucap
do_script_tempck
do_script_tempwatch
do_script_masternic
do_script_rclocal
do_script_copies
do_tty
do_unbrand "${thishost}"
do_admin_user
do_sshd

echo
echo "Done."
echo
echo "Run init-disk to setup the first disk."
echo

